// PaintDlg.cpp : implementation file
//

#include "stdafx.h"
#include "Graphics.h"
#include "PaintDlg.h"
#include "GraphicsDlg.h"

const COLORREF CPaintDlg::m_crColors[8] = {
	RGB(   0,   0,   0),    // Black
    RGB(   0,   0, 255),    // Blue
    RGB(   0, 255,   0),    // Green
    RGB(   0, 255, 255),    // Cyan
    RGB( 255,   0,   0),    // Red
    RGB( 255,   0, 255),    // Magenta
    RGB( 255, 255,   0),    // Yellow
    RGB( 255, 255, 255)     // White
};

// CPaintDlg dialog

IMPLEMENT_DYNAMIC(CPaintDlg, CDialog)
CPaintDlg::CPaintDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CPaintDlg::IDD, pParent)
{
}

CPaintDlg::~CPaintDlg()
{
}

void CPaintDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
}


BEGIN_MESSAGE_MAP(CPaintDlg, CDialog)
	ON_WM_PAINT()
END_MESSAGE_MAP()


// CPaintDlg message handlers

void CPaintDlg::DrawLine(CPaintDC* pdc, int iColor)
{
    // Declare and create the pens
    CPen pnSolidPen (PS_SOLID, 1, m_crColors[iColor]);
    CPen pnDotPen (PS_DOT, 1, m_crColors[iColor]);
    CPen pnDashPen (PS_DASH, 1, m_crColors[iColor]);
    CPen pnDashDotPen (PS_DASHDOT, 1, m_crColors[iColor]);
    CPen pnDashDotDotPen (PS_DASHDOTDOT, 1, m_crColors[iColor]);
    CPen pnNullPen (PS_NULL, 1, m_crColors[iColor]);
    CPen pnInsidePen (PS_INSIDEFRAME, 1, m_crColors[iColor]);
    
    // Get the drawing area
    CRect rRect;
    GetClientRect(rRect);
    rRect.NormalizeRect();

    // Calculate the distance between each of the lines
    CPoint ptStart;
    CPoint ptEnd;
    int iDist = rRect.Height() / 8;
    CPen* pOldPen = NULL;
    // Specify the starting points
    ptStart.y = rRect.top;
    ptStart.x = rRect.left;
    ptEnd.y = ptStart.y;
    ptEnd.x = rRect.right;
    int i;
    // Loop through the different pens
    for (i = 0; i < 7; i++)
    {
        // Which pen are we on?
        switch (i)
        {
        case 0:    // Solid
            pOldPen = pdc->SelectObject(&pnSolidPen);
            break;
        case 1:    // Dot
            pdc->SelectObject(&pnDotPen);
            break;
        case 2:    // Dash
            pdc->SelectObject(&pnDashPen);
            break;
        case 3:    // Dash Dot
            pdc->SelectObject(&pnDashDotPen);
            break;
        case 4:    // Dash Dot Dot
            pdc->SelectObject(&pnDashDotDotPen);
            break;
        case 5:    // Null
            pdc->SelectObject(&pnNullPen);
            break;
        case 6:    // Inside
            pdc->SelectObject(&pnInsidePen);
            break;
        }
        // Move down to the next position
        ptStart.y = ptStart.y + iDist;
        ptEnd.y = ptStart.y;
        // Draw the line
        pdc->MoveTo(ptStart);
        pdc->LineTo(ptEnd);
    }
    // Select the original pen
    pdc->SelectObject(pOldPen);
}

void CPaintDlg::OnPaint()
{
	CPaintDC dc(this); // device context for painting
	// TODO: Add your message handler code here
	// Get a pointer to the parent window
    CGraphicsDlg *pWnd = (CGraphicsDlg*)GetParent();
    // Do we have a valid pointer?
    if (pWnd)
    {
        // Is the tool a bitmap?
        if (pWnd->m_iTool == 2)
        {
	        // Is there a bitmap selected and loaded?
            if (pWnd->m_strBitmap.GetLength() >0)
                // Display it
                ShowBitmap(&dc);
        }
        else    // No, we're drawing a shape
        {
            // Are we drawing a line?
            if (pWnd->m_iShape == 0)
                DrawLine(&dc, pWnd->m_iColor);
			else    // We're drawing a ellipse or rectangle
                DrawRegion(&dc, pWnd->m_iColor, pWnd->m_iTool, pWnd->m_iShape);
        }
    }
    // Do not call CDialog::OnPaint() for painting messages
}

void CPaintDlg::DrawRegion(CPaintDC* pdc, int iColor, int iTool, int iShape)
{
    // Declare and create the pens
    CPen pnSolidPen (PS_SOLID, 1, m_crColors[iColor]);
    CPen pnDotPen (PS_DOT, 1, m_crColors[iColor]);
    CPen pnDashPen (PS_DASH, 1, m_crColors[iColor]);
    CPen pnDashDotPen (PS_DASHDOT, 1, m_crColors[iColor]);
    CPen pnDashDotDotPen (PS_DASHDOTDOT, 1, m_crColors[iColor]);
    CPen pnNullPen (PS_NULL, 1, m_crColors[iColor]);
    CPen pnInsidePen (PS_INSIDEFRAME, 1, m_crColors[iColor]);

    // Declare and create the brushes
    CBrush brSolidBrush(m_crColors[iColor]);
    CBrush brBDiagBrush(HS_BDIAGONAL, m_crColors[iColor]);
    CBrush brCrossBrush(HS_CROSS, m_crColors[iColor]);
    CBrush brDiagCrossBrush(HS_DIAGCROSS, m_crColors[iColor]);
    CBrush brFDiagBrush(HS_FDIAGONAL, m_crColors[iColor]);
    CBrush brHorizBrush(HS_HORIZONTAL, m_crColors[iColor]);
    CBrush brVertBrush(HS_VERTICAL, m_crColors[iColor]);
    CBrush brNullBrush(RGB(208, 208, 208));

    // Calculate the size the the drawing regions
    CRect rRect;
    GetClientRect(rRect);
    rRect.NormalizeRect();
    int iVert = rRect.Height() / 2;
    int iHeight = iVert - 10;
    int iHorz = rRect.Width() / 4;
    int iWidth = iHorz - 10;
    CRect rDrawRect;
    CPen *pOldPen = NULL;
    CBrush *pOldBrush = NULL;
    // Loop through all of the brushes and pens
    for (int i = 0; i < 7; i++)
    {
        if (i == 0)
        {
            // Determine the location for this figure.
            // Start the first row
            rDrawRect.top = rRect.top + 5;
            rDrawRect.left = rRect.left + 5;
            rDrawRect.bottom = rDrawRect.top + iHeight;
            rDrawRect.right = rDrawRect.left + iWidth;
        }
        else if (i == 4)
        {
            // Determine the location for this figure.
            // Start the second row
            rDrawRect.top = rDrawRect.top + iVert;
            rDrawRect.left = rRect.left + 5;
            rDrawRect.bottom = rDrawRect.top + iHeight;
            rDrawRect.right = rDrawRect.left + iWidth;
        }
        else
        {
            // Determine the location for this figure.
            rDrawRect.left = rDrawRect.left + iHorz;
            rDrawRect.right = rDrawRect.left + iWidth;
        }
        switch (i)
        {
        case 0:    // Solid
            // Select the appropriate pen and brush
            pOldPen = pdc->SelectObject(&pnSolidPen);
            pOldBrush = pdc->SelectObject(&brSolidBrush);
            break;
        case 1:    // Dot - Back Diagonal
            // Select the appropriate pen and brush
            pdc->SelectObject(&pnDotPen);
            pdc->SelectObject(&brBDiagBrush);
            break;
        case 2:    // Dash - Cross Brush
            // Select the appropriate pen and brush
            pdc->SelectObject(&pnDashPen);
            pdc->SelectObject(&brCrossBrush);
            break;
        case 3:    // Dash Dot - Diagonal Cross
            // Select the appropriate pen and brush
            pdc->SelectObject(&pnDashDotPen);
            pdc->SelectObject(&brDiagCrossBrush);
            break;
        case 4:    // Dash Dot Dot - Forward Diagonal
            // Select the appropriate pen and brush
            pdc->SelectObject(&pnDashDotDotPen);
            pdc->SelectObject(&brFDiagBrush);
            break;
        case 5:    // Null - Horizontal
            // Select the appropriate pen and brush
            pdc->SelectObject(&pnNullPen);
            pdc->SelectObject(&brHorizBrush);
            break;
        case 6:    // Inside - Vertical
            // Select the appropriate pen and brush
            pdc->SelectObject(&pnInsidePen);
            pdc->SelectObject(&brVertBrush);
            break;
        }
        // Which tool are we using?
        if (iTool == 0)
            pdc->SelectObject(brNullBrush);
        else
            pdc->SelectObject(pnNullPen);
        // Which shape are we drawing?
        if (iShape == 1)
            pdc->Ellipse(rDrawRect);
        else
            pdc->Rectangle(rDrawRect);
    }
    // Reset the original brush and pen
    pdc->SelectObject(pOldBrush);
    pdc->SelectObject(pOldPen);
}

void CPaintDlg::ShowBitmap(CPaintDC* pdc)
{
    // Convert the pointer to a pointer to the main dialog class
    CGraphicsDlg *pWnd = (CGraphicsDlg*)GetParent();
//    BITMAP bm;
    CDC dcMem;
    CRect rRect;

//    // Get the loaded bitmap
//    pWnd->m_bmpBitmap.GetBitmap(&bm);
//    // Create a device context to load the bitmap into
//    dcMem.CreateCompatibleDC(pdc);
//    // Select the bitmap into the compatible device context
//    dcMem.SelectObject(pWnd->m_bmpBitmap);
    // Get the display area available
    GetClientRect(rRect);
    rRect.NormalizeRect();
    // Copy and resize the bitmap to the dialog window
//    pdc->StretchBlt(10, 10, (rRect.Width() - 20), (rRect.Height() - 20), 
//        &dcMem, 0, 0, bm.bmWidth, bm.bmHeight, SRCCOPY);
	pWnd->m_bmpBitmap.Draw(pdc->m_hDC, rRect);
}
