// SDISquigDoc.cpp : implementation of the CSDISquigDoc class
//

#include "stdafx.h"
#include "SDISquig.h"

#include "SDISquigDoc.h"
#include "MainFrm.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


// CSDISquigDoc

IMPLEMENT_DYNCREATE(CSDISquigDoc, CDocument)

BEGIN_MESSAGE_MAP(CSDISquigDoc, CDocument)
	ON_COMMAND(ID_COLOR_BLACK, OnColorBlack)
	ON_UPDATE_COMMAND_UI(ID_COLOR_BLACK, OnUpdateColorBlack)
	ON_COMMAND(ID_COLOR_BLUE, OnColorBlue)
	ON_UPDATE_COMMAND_UI(ID_COLOR_BLUE, OnUpdateColorBlue)
	ON_COMMAND(ID_COLOR_GREEN, OnColorGreen)
	ON_UPDATE_COMMAND_UI(ID_COLOR_GREEN, OnUpdateColorGreen)
	ON_COMMAND(ID_COLOR_CYAN, OnColorCyan)
	ON_UPDATE_COMMAND_UI(ID_COLOR_CYAN, OnUpdateColorCyan)
	ON_COMMAND(ID_COLOR_RED, OnColorRed)
	ON_UPDATE_COMMAND_UI(ID_COLOR_RED, OnUpdateColorRed)
	ON_COMMAND(ID_COLOR_MAGENTA, OnColorMagenta)
	ON_UPDATE_COMMAND_UI(ID_COLOR_MAGENTA, OnUpdateColorMagenta)
	ON_COMMAND(ID_COLOR_YELLOW, OnColorYellow)
	ON_UPDATE_COMMAND_UI(ID_COLOR_YELLOW, OnUpdateColorYellow)
	ON_COMMAND(ID_COLOR_WHITE, OnColorWhite)
	ON_UPDATE_COMMAND_UI(ID_COLOR_WHITE, OnUpdateColorWhite)
	ON_COMMAND(ID_WIDTH_VTHIN, OnWidthVthin)
	ON_UPDATE_COMMAND_UI(ID_WIDTH_VTHIN, OnUpdateWidthVthin)
	ON_COMMAND(ID_WIDTH_THIN, OnWidthThin)
	ON_UPDATE_COMMAND_UI(ID_WIDTH_THIN, OnUpdateWidthThin)
	ON_COMMAND(ID_WIDTH_MEDIUM, OnWidthMedium)
	ON_UPDATE_COMMAND_UI(ID_WIDTH_MEDIUM, OnUpdateWidthMedium)
	ON_COMMAND(ID_WIDTH_THICK, OnWidthThick)
	ON_UPDATE_COMMAND_UI(ID_WIDTH_THICK, OnUpdateWidthThick)
	ON_COMMAND(ID_WIDTH_VTHICK, OnWidthVthick)
	ON_UPDATE_COMMAND_UI(ID_WIDTH_VTHICK, OnUpdateWidthVthick)
	ON_UPDATE_COMMAND_UI(ID_INDICATOR_COLOR, OnUpdateIndicatorColor)
END_MESSAGE_MAP()

const COLORREF CSDISquigDoc::m_crColors[8] = {
    RGB(   0,   0,   0),    // Black
    RGB(   0,   0, 255),    // Blue
    RGB(   0, 255,   0),    // Green
    RGB(   0, 255, 255),    // Cyan
    RGB( 255,   0,   0),    // Red
    RGB( 255,   0, 255),    // Magenta
    RGB( 255, 255,   0),    // Yellow
    RGB( 255, 255, 255)     // White
};

// CSDISquigDoc construction/destruction

CSDISquigDoc::CSDISquigDoc()
: m_nColor(0)
, m_nWidth(0)
{
	// TODO: add one-time construction code here

}

CSDISquigDoc::~CSDISquigDoc()
{
}

BOOL CSDISquigDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;

	// TODO: add reinitialization code here
	// (SDI documents will reuse this document)
    // Initialize the color to black
    m_nColor = 0;
	// Initialize the width to thin
    m_nWidth = ID_WIDTH_VTHIN - ID_WIDTH_VTHIN;

	return TRUE;
}




// CSDISquigDoc serialization

void CSDISquigDoc::Serialize(CArchive& ar)
{
	// Pass the serialization on to the object array
    m_oaLines.Serialize(ar);
}


// CSDISquigDoc diagnostics

#ifdef _DEBUG
void CSDISquigDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CSDISquigDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG


// CSDISquigDoc commands

CLine* CSDISquigDoc::AddLine(CPoint ptFrom, CPoint ptTo)
{
	static UINT nWidths[5] = { 1, 8, 16, 24, 32};
	CLine* pLine = NULL;
	try
    {
        // Create a new CLine object
        pLine = new CLine(ptFrom, ptTo, m_crColors[m_nColor], nWidths[m_nWidth]);
        // Add the new line to the object array
        m_oaLines.Add(pLine);
        // Mark the document as dirty
        SetModifiedFlag();
    }
    // Did we run into a memory exception?
    catch (CMemoryException* perr)
    {
        // Display a message for the user, giving him or her the
        // bad news
        AfxMessageBox("Out of memory", MB_ICONSTOP | MB_OK);
        // Did we create a line object?
        if (pLine)
        {
            // Delete it
            delete pLine;
            pLine = NULL;
        }
        // Delete the exception object
        perr->Delete();
    }
    return pLine;
}

int CSDISquigDoc::GetLineCount(void)
{
	// Return the array count
	return (int)m_oaLines.GetCount();
}

CLine* CSDISquigDoc::GetLine(int iIndex)
{
	// Return a pointer to the line object
    // at the specified point in the object array
    return (CLine*)m_oaLines[iIndex];
}

void CSDISquigDoc::DeleteContents()
{
	// TODO: Add your specialized code here and/or call the base class
	// Get the number of lines in the object array
    int iCount = (int)m_oaLines.GetCount();
    int iPos;

    try
    {
        // Are there any objects in the array?
        if (iCount)
        {
            //Loop through the array, deleting each object
            for (iPos = 0; iPos < iCount; iPos++)
                delete (CLine*)m_oaLines.GetAt(iPos);
            // Reset the array
            m_oaLines.RemoveAll();
        }
    }
    catch (CMemoryException* perr)
    {
        // Display a message for the user, giving him or her the
        // bad news
        perr->ReportError();
        // Delete the exception object
        perr->Delete();
    }
	CDocument::DeleteContents();
}

UINT CSDISquigDoc::GetColor(void)
{
	// Return the current color
    return ID_COLOR_BLACK + m_nColor;
}

void CSDISquigDoc::OnColorBlack()
{
	// TODO: Add your command handler code here
	// Set the current color to black
    m_nColor = ID_COLOR_BLACK - ID_COLOR_BLACK;
}

void CSDISquigDoc::OnUpdateColorBlack(CCmdUI *pCmdUI)
{
	// TODO: Add your command update UI handler code here
	// Determine if the Black menu entry should be checked
    pCmdUI->Enable();
    pCmdUI->SetCheck(GetColor() == ID_COLOR_BLACK ? 1 : 0);
}

void CSDISquigDoc::OnColorBlue()
{
	// TODO: Add your command handler code here
	// Set the current color to blue
    m_nColor = ID_COLOR_BLUE - ID_COLOR_BLACK;
}

void CSDISquigDoc::OnUpdateColorBlue(CCmdUI *pCmdUI)
{
	// TODO: Add your command update UI handler code here
	// Determine if the Blue menu entry should be checked
    pCmdUI->Enable();
    pCmdUI->SetCheck(GetColor() == ID_COLOR_BLUE ? 1 : 0);
}

void CSDISquigDoc::OnColorGreen()
{
	// TODO: Add your command handler code here
	// Set the current color to green
    m_nColor = ID_COLOR_GREEN - ID_COLOR_BLACK;
}

void CSDISquigDoc::OnUpdateColorGreen(CCmdUI *pCmdUI)
{
	// TODO: Add your command update UI handler code here
	// Determine if the Green menu entry should be checked
    pCmdUI->Enable();
    pCmdUI->SetCheck(GetColor() == ID_COLOR_GREEN ? 1 : 0);
}

void CSDISquigDoc::OnColorCyan()
{
	// TODO: Add your command handler code here
	// Set the current color to cyan
    m_nColor = ID_COLOR_CYAN - ID_COLOR_BLACK;
}

void CSDISquigDoc::OnUpdateColorCyan(CCmdUI *pCmdUI)
{
	// TODO: Add your command update UI handler code here
	// Determine if the Cyan menu entry should be checked
    pCmdUI->Enable();
    pCmdUI->SetCheck(GetColor() == ID_COLOR_CYAN ? 1 : 0);
}

void CSDISquigDoc::OnColorRed()
{
	// TODO: Add your command handler code here
	// Set the current color to red
    m_nColor = ID_COLOR_RED - ID_COLOR_BLACK;
}

void CSDISquigDoc::OnUpdateColorRed(CCmdUI *pCmdUI)
{
	// TODO: Add your command update UI handler code here
	// Determine if the Red menu entry should be checked
    pCmdUI->Enable();
    pCmdUI->SetCheck(GetColor() == ID_COLOR_RED ? 1 : 0);
}

void CSDISquigDoc::OnColorMagenta()
{
	// TODO: Add your command handler code here
	// Set the current color to magenta
    m_nColor = ID_COLOR_MAGENTA - ID_COLOR_BLACK;
}

void CSDISquigDoc::OnUpdateColorMagenta(CCmdUI *pCmdUI)
{
	// TODO: Add your command update UI handler code here
	// Determine if the Magenta menu entry should be checked
    pCmdUI->Enable();
    pCmdUI->SetCheck(GetColor() == ID_COLOR_MAGENTA ? 1 : 0);
}

void CSDISquigDoc::OnColorYellow()
{
	// TODO: Add your command handler code here
	// Set the current color to yellow
    m_nColor = ID_COLOR_YELLOW - ID_COLOR_BLACK;
}

void CSDISquigDoc::OnUpdateColorYellow(CCmdUI *pCmdUI)
{
	// TODO: Add your command update UI handler code here
	// Determine if the Yellow menu entry should be checked
    pCmdUI->Enable();
    pCmdUI->SetCheck(GetColor() == ID_COLOR_YELLOW ? 1 : 0);
}

void CSDISquigDoc::OnColorWhite()
{
	// TODO: Add your command handler code here
	// Set the current color to white
    m_nColor = ID_COLOR_WHITE - ID_COLOR_BLACK;
}

void CSDISquigDoc::OnUpdateColorWhite(CCmdUI *pCmdUI)
{
	// TODO: Add your command update UI handler code here
	// Determine if the White menu entry should be checked
    pCmdUI->Enable();
    pCmdUI->SetCheck(GetColor() == ID_COLOR_WHITE ? 1 : 0);
}

UINT CSDISquigDoc::GetWidth(void)
{
    // Return the current width
    return ID_WIDTH_VTHIN + m_nWidth;
}

void CSDISquigDoc::OnWidthVthin()
{
	// TODO: Add your command handler code here
	// Set the current width to very thin
    m_nWidth = ID_WIDTH_VTHIN - ID_WIDTH_VTHIN;
}

void CSDISquigDoc::OnUpdateWidthVthin(CCmdUI *pCmdUI)
{
	// TODO: Add your command update UI handler code here
	// Determine if the Very Thin menu entry should be checked
    pCmdUI->SetCheck(GetWidth() == ID_WIDTH_VTHIN ? 1 : 0);
	// Update the combo box on the color toolbar
    UpdateColorbar(0);
}

void CSDISquigDoc::OnWidthThin()
{
	// TODO: Add your command handler code here
	// Set the current width to thin
    m_nWidth = ID_WIDTH_THIN - ID_WIDTH_VTHIN;
}

void CSDISquigDoc::OnUpdateWidthThin(CCmdUI *pCmdUI)
{
	// TODO: Add your command update UI handler code here
	// Determine if the Thin menu entry should be checked
    pCmdUI->SetCheck(GetWidth() == ID_WIDTH_THIN ? 1 : 0);
	// Update the combo box on the color toolbar
    UpdateColorbar(1);
}

void CSDISquigDoc::OnWidthMedium()
{
	// TODO: Add your command handler code here
	// Set the current width to medium
    m_nWidth = ID_WIDTH_MEDIUM - ID_WIDTH_VTHIN;
}

void CSDISquigDoc::OnUpdateWidthMedium(CCmdUI *pCmdUI)
{
	// TODO: Add your command update UI handler code here
	// Determine if the Medium menu entry should be checked
    pCmdUI->SetCheck(GetWidth() == ID_WIDTH_MEDIUM ? 1 : 0);
	// Update the combo box on the color toolbar
    UpdateColorbar(2);
}

void CSDISquigDoc::OnWidthThick()
{
	// TODO: Add your command handler code here
	// Set the current width to thick
    m_nWidth = ID_WIDTH_THICK - ID_WIDTH_VTHIN;
}

void CSDISquigDoc::OnUpdateWidthThick(CCmdUI *pCmdUI)
{
	// TODO: Add your command update UI handler code here
	// Determine if the Thick menu entry should be checked
    pCmdUI->SetCheck(GetWidth() == ID_WIDTH_THICK ? 1 : 0);
	// Update the combo box on the color toolbar
    UpdateColorbar(3);
}

void CSDISquigDoc::OnWidthVthick()
{
	// TODO: Add your command handler code here
	// Set the current width to very thick
    m_nWidth = ID_WIDTH_VTHICK - ID_WIDTH_VTHIN;
}

void CSDISquigDoc::OnUpdateWidthVthick(CCmdUI *pCmdUI)
{
	// TODO: Add your command update UI handler code here
	// Determine if the Very Thick menu entry should be checked
    pCmdUI->SetCheck(GetWidth() == ID_WIDTH_VTHICK ? 1 : 0);
	// Update the combo box on the color toolbar
    UpdateColorbar(4);
}

void CSDISquigDoc::SetWidth(unsigned int nWidth)
{
	// Set the current width
    m_nWidth = nWidth;
}

void CSDISquigDoc::UpdateColorbar(int iIndex)
{
	// Get the position of the first view
    POSITION pos = GetFirstViewPosition();
    // Did we get a valid position?
    if (pos != NULL)
    {
        // Get a pointer to the view in that position
        CView* pView = GetNextView(pos);
        // Do we have a valid pointer to the view?
        if (pView)
        {
            // Get a pointer to the frame through the view
            CMainFrame* pFrame = (CMainFrame*)pView->GetTopLevelFrame();
            // Did we get a pointer to the frame?
            if (pFrame)
                // Update the combo box on the color toolbar
                // through the frame
                pFrame->UpdateWidthCB(iIndex);
        }
    }
}

void CSDISquigDoc::OnUpdateIndicatorColor(CCmdUI* pCmdUI)
{
	CString strColor;

	TRACE("OnUpdateIndicatorColor\n");
    // What is the current color?
    switch (m_nColor)
    {
    case 0:    // Black
        strColor = "BLACK";
        break;
    case 1:    // Blue
        strColor = "BLUE";
        break;
    case 2:    // Green
        strColor = "GREEN";
        break;
    case 3:    // Cyan
        strColor = "CYAN";
        break;
    case 4:    // Red
        strColor = "RED";
        break;
    case 5:    // Magenta
        strColor = "MAGENTA";
        break;
    case 6:    // Yellow
        strColor = "YELLOW";
        break;
    case 7:    // White
        strColor = "WHITE";
        break;
    }
    // Enable the status bar pane
    pCmdUI->Enable(TRUE);
    // Set the text of the status bar pane 
    // to the current color
    pCmdUI->SetText(strColor);
}
