// SerializeDoc.cpp : implementation of the CSerializeDoc class
//

#include "stdafx.h"
#include "Serialize.h"

#include "SerializeDoc.h"
#include "SerializeView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


// CSerializeDoc

IMPLEMENT_DYNCREATE(CSerializeDoc, CDocument)

BEGIN_MESSAGE_MAP(CSerializeDoc, CDocument)
END_MESSAGE_MAP()


// CSerializeDoc construction/destruction

CSerializeDoc::CSerializeDoc()
: m_iCurPosition(0)
{
	// TODO: add one-time construction code here

}

CSerializeDoc::~CSerializeDoc()
{
}

BOOL CSerializeDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;

	// TODO: add reinitialization code here
	// (SDI documents will reuse this document)
	// If unable to add a new record, return FALSE
    if (!AddNewRecord())
        return FALSE;

    // Get a pointer to the view
    POSITION pos = GetFirstViewPosition();
    CSerializeView* pView = (CSerializeView*)GetNextView(pos);
    // Tell the view that it's got a new data set
    if (pView)
        pView->NewDataSet();

	return TRUE;
}




// CSerializeDoc serialization

void CSerializeDoc::Serialize(CArchive& ar)
{
	// Pass the serialization on to the object array
    m_oaPeople.Serialize(ar);
}


// CSerializeDoc diagnostics

#ifdef _DEBUG
void CSerializeDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CSerializeDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG


// CSerializeDoc commands

CPerson* CSerializeDoc::AddNewRecord(void)
{
	// Create a new CPerson object
    CPerson *pPerson = NULL;
    try
    {
        pPerson = new CPerson();
        // Add the new person to the object array
        m_oaPeople.Add(pPerson);
        // Mark the document as dirty
        SetModifiedFlag();
        // Set the new position mark
        m_iCurPosition = (m_oaPeople.GetSize() - 1);
    }
    // Did we run into a memory exception?
    catch (CMemoryException* perr)
    {
        // Display a message for the user, giving them the
        // bad news
        AfxMessageBox("Out of memory", MB_ICONSTOP | MB_OK);
        // Did we create a person object?
        if (pPerson)
        {
            // Delete it
            delete pPerson;
            pPerson = NULL;
        }
        // Delete the exception object
        perr->Delete();
    }
    return pPerson;
}

int CSerializeDoc::GetTotalRecords(void)
{
	// Return the array count
    return m_oaPeople.GetSize();
}

int CSerializeDoc::GetCurRecordNbr(void)
{
	// Return the current position
    return (m_iCurPosition + 1);
}

CPerson* CSerializeDoc::GetCurRecord(void)
{
	// Are we editing a valid record number?
    if (m_iCurPosition >= 0)
        // Yes, return the current record
        return (CPerson*)m_oaPeople.GetAt(m_iCurPosition);
    else
        // No, return NULL
        return NULL;
}

CPerson* CSerializeDoc::GetFirstRecord(void)
{
	// Are there any records in the array?
    if (m_oaPeople.GetSize() > 0)
    {
        // Yes, move to position 0
        m_iCurPosition = 0;
        // Return the record in position 0
        return (CPerson*)m_oaPeople.GetAt(0);
    }
    else
        // No records, return NULL
        return NULL;
}

CPerson* CSerializeDoc::GetNextRecord(void)
{
	// After incrementing the position marker, are we
    // past the end of the array?
    if (++m_iCurPosition < m_oaPeople.GetSize())
        // No, return the record at the new current position
        return (CPerson*)m_oaPeople.GetAt(m_iCurPosition);
    else
        // Yes, add a new record
        return AddNewRecord();
}

CPerson* CSerializeDoc::GetPrevRecord(void)
{
	// Are there any records in the array?
    if (m_oaPeople.GetSize() > 0)
    {
        // Once we decrement the current position,
        // are we below position 0?
        if (--m_iCurPosition < 0)
            // If so, set the record to position 0
            m_iCurPosition = 0;
        // Return the record at the new current position
        return (CPerson*)m_oaPeople.GetAt(m_iCurPosition);
    }
    else
        // No records, return NULL
        return NULL;
}

CPerson* CSerializeDoc::GetLastRecord(void)
{
	// Are there any records in the array?
    if (m_oaPeople.GetSize() > 0)
    {
        // Move to the last position in the array
        m_iCurPosition = m_oaPeople.GetUpperBound();
        // Return the record in this position
        return (CPerson*)m_oaPeople.GetAt(m_iCurPosition);
    }
    else
        // No records, return NULL
        return NULL;
}

void CSerializeDoc::DeleteContents()
{
	// TODO: Add your specialized code here and/or call the base class
	// Get the number of people in the object array
    int iCount = m_oaPeople.GetSize();
    int iPos;

    // Loop through the array, deleting each object
    for (iPos = 0; iPos < iCount; iPos++)
        delete (CPerson*)m_oaPeople.GetAt(iPos);
    // Reset the array
    m_oaPeople.RemoveAll();
	CDocument::DeleteContents();
}

BOOL CSerializeDoc::OnOpenDocument(LPCTSTR lpszPathName)
{
	if (!CDocument::OnOpenDocument(lpszPathName))
		return FALSE;

	// TODO:  Add your specialized creation code here
	// Get a pointer to the view
    POSITION pos = GetFirstViewPosition();
    CSerializeView* pView = (CSerializeView*)GetNextView(pos);
    // Tell the view that it's got a new data set
    if (pView)
        pView->NewDataSet();

	return TRUE;
}
