#include "StdAfx.h"
#include "modart.h"
#include "line.h"

void NewLine(CRect* pRect, CObArray* poaLines)
{
    int iNumLines;
    int iCurLine;
    int iCurColor;
    UINT nCurWidth;
    CPoint ptTo;
    CPoint ptFrom;

    static COLORREF crColors[8] = {
        RGB(   0,   0,   0),    // Black
        RGB(   0,   0, 255),    // Blue
        RGB(   0, 255,   0),    // Green
        RGB(   0, 255, 255),    // Cyan
        RGB( 255,   0,   0),    // Red
        RGB( 255,   0, 255),    // Magenta
        RGB( 255, 255,   0),    // Yellow
        RGB( 255, 255, 255)     // White
    };

    // Normalize the rectangle before determining the width and height
    pRect->NormalizeRect();
    // get the area width and height
    int iWidth = pRect->Width();
    int iHeight = pRect->Height();

    // Determine the number of parts to this squiggle
    iNumLines = rand() % 100;
    // Are there any parts to this squiggle?
    if (iNumLines > 0)
    {
        // Determine the color
        iCurColor = rand() % 8;
        // Determine the pen width
        nCurWidth = (rand() % 8) + 1;
        // Determine the starting point for the squiggle
	    ptFrom.x = (rand() % iWidth) + pRect->left;
        ptFrom.y = (rand() % iHeight) + pRect->top;
        // Loop through the number of segments
        for (iCurLine = 0; iCurLine < iNumLines; iCurLine++)
        {
            // Determine the end point of the segment
            ptTo.x = ((rand() % 20) - 10) + ptFrom.x;
            ptTo.y = ((rand() % 20) - 10) + ptFrom.y;
            // Create a new CLine object
            CLine *pLine = NULL;
            try
            {
                pLine = new CLine(ptFrom, ptTo, crColors[iCurColor],
                    nCurWidth);
                // Add the new line to the object array
                poaLines->Add(pLine);
            }
            // Did we run into a memory exception?
            catch (CMemoryException* perr)
            {
                // Display a message for the user, giving him the
                // bad news
                AfxMessageBox("Out of memory", MB_ICONSTOP | MB_OK);
                // Did we create a line object?
                if (pLine)
                {
                    // Delete it
                    delete pLine;
                    pLine = NULL;
                }
                // Delete the exception object
                perr->Delete();
            }
            // Set the starting point to the end point
            ptFrom = ptTo;
        }
    }
}

extern "C" void PASCAL EXPORT ModArtNewDrawing(CRect* pRect, CObArray* poaLines)
{
    AFX_MANAGE_STATE(AfxGetStaticModuleState());
    // normal function body here
	int iNumLines;
    int iCurLine;

    // Make sure that we have a valid pointer
    if (!poaLines) return;

	// Initialize the random number generator
    srand((unsigned)time(NULL));
    // Determine how many lines to create
    iNumLines = rand() % 10;
    // Are there any lines to create?
    if (iNumLines > 0)
    {
        // Loop through the number of lines
        for (iCurLine = 0; iCurLine < iNumLines; iCurLine++)
        {
            // Create the new line
            NewLine(pRect, poaLines);
        }
    }
}

extern "C" void PASCAL EXPORT ModArtDraw(CDC* pDC, CObArray* poaLines)
{
    AFX_MANAGE_STATE(AfxGetStaticModuleState());
    // Make sure that we have a valid pointer
    if (!poaLines) return;

	// Get the number of lines in the object array
    int iCount = poaLines->GetSize();
	int iPos;

    // Are there any objects in the array?
    if (iCount)
    {
        // Loop through the array, deleting each object
        for (iPos = 0; iPos < iCount; iPos++)
            ((CLine*)poaLines->GetAt(iPos))->Draw(pDC);
    }
}

extern "C" void PASCAL EXPORT ModArtSerialize(CArchive& ar, CObArray* poaLines)
{
    AFX_MANAGE_STATE(AfxGetStaticModuleState());
    // Make sure that we have a valid pointer
    if (!poaLines) return;
    // Pass the archive object on to the array
    poaLines->Serialize(ar);
}

extern "C" void PASCAL EXPORT ModArtClearDrawing(CObArray* poaLines)
{
    AFX_MANAGE_STATE(AfxGetStaticModuleState());
    // Make sure that we have a valid pointer
    if (!poaLines) return;

	// Get the number of lines in the object array
    int iCount = poaLines->GetSize();
	int iPos;

    // Are there any objects in the array?
    // Loop through the array, deleting each object
    for (iPos = 0; iPos < iCount; iPos++)
        delete poaLines->GetAt(iPos);
    // Reset the array
    poaLines->RemoveAll();
}
