#include "StdAfx.h"
#include "modart.h"
#include "line.h"

CModArt::CModArt(void)
: m_iLength(200)
, m_iSegments(50)
{
    // Initialize the random number generator
    srand((unsigned)time(NULL));
}

CModArt::~CModArt(void)
{
}

void CModArt::SetRect(CRect rDrawArea)
{
    // Set the drawing area rectangle
    m_rDrawArea = rDrawArea;
}

void CModArt::NewLine(void)
{
    int iNumLines;
    int iCurLine;
    int iCurColor;
    UINT nCurWidth;
    CPoint ptTo;
    CPoint ptFrom;

    static COLORREF crColors[8] = {
        RGB(   0,   0,   0),    // Black
        RGB(   0,   0, 255),    // Blue
        RGB(   0, 255,   0),    // Green
        RGB(   0, 255, 255),    // Cyan
        RGB( 255,   0,   0),    // Red
        RGB( 255,   0, 255),    // Magenta
        RGB( 255, 255,   0),    // Yellow
        RGB( 255, 255, 255)     // White
    };

    // Normalize the rectangle before determining the width and height
    m_rDrawArea.NormalizeRect();
    // get the area width and height
    int iWidth = m_rDrawArea.Width();
    int iHeight = m_rDrawArea.Height();

    // Determine the number of parts to this squiggle
    iNumLines = rand() % m_iLength;
    // Are there any parts to this squiggle?
    if (iNumLines > 0)
    {
        // Determine the color
        iCurColor = rand() % 8;
        // Determine the pen width
        nCurWidth = (rand() % 8) + 1;
        // Determine the starting point for the squiggle
        ptFrom.x = (rand() % iWidth) + m_rDrawArea.left;
        ptFrom.y = (rand() % iHeight) + m_rDrawArea.top;
        // Loop through the number of segments
        for (iCurLine = 0; iCurLine < iNumLines; iCurLine++)
        {
            // Determine the end point of the segment
            ptTo.x = ((rand() % 20) - 10) + ptFrom.x;
            ptTo.y = ((rand() % 20) - 10) + ptFrom.y;
            // Create a new CLine object
            CLine *pLine = NULL;
            try
            {
                pLine = new CLine(ptFrom, ptTo, crColors[iCurColor],
                    nCurWidth);
                // Add the new line to the object array
                m_oaLines.Add(pLine);
            }
            // Did we run into a memory exception?
            catch (CMemoryException* perr)
            {
                // Display a message for the user, giving him the
                // bad news
                AfxMessageBox("Out of memory", MB_ICONSTOP | MB_OK);
                // Did we create a line object?
                if (pLine)
                {
                    // Delete it
                    delete pLine;
                    pLine = NULL;
                }
                // Delete the exception object
                perr->Delete();
            }
            // Set the starting point to the end point
            ptFrom = ptTo;
        }
    }
}

void CModArt::NewDrawing(void)
{
    int iNumLines;
    int iCurLine;

    // Determine how many lines to create
    iNumLines = rand() % m_iSegments;
    // Are there any lines to create?
    if (iNumLines > 0)
    {
        // Loop through the number of lines
        for (iCurLine = 0; iCurLine < iNumLines; iCurLine++)
        {
            // Create the new line
            NewLine();
        }
    }
}

void CModArt::Draw(CDC* pDC)
{
    // Get the number of lines in the object array
    int iCount = m_oaLines.GetSize();
    int iPos;

    // Are there any objects in the array?
    if (iCount)
    {
        // Loop through the array, deleting each object
        for (iPos = 0; iPos < iCount; iPos++)
            ((CLine*)m_oaLines.GetAt(iPos))->Draw(pDC);
    }
}

void CModArt::Serialize(CArchive& ar)
{
    // Pass the archive object on to the array
    m_oaLines.Serialize(ar);
}

void CModArt::ClearDrawing(void)
{
    // Get the number of lines in the object array
    int iCount = m_oaLines.GetSize();
    int iPos;

    // Are there any objects in the array?
    // Loop through the array, deleting each object
    for (iPos = 0; iPos < iCount; iPos++)
        delete m_oaLines.GetAt(iPos);
    // Reset the array
    m_oaLines.RemoveAll();
}

int CModArt::GetLength(void)
{
	// Return the current value for the m_iLength variable
	return m_iLength;
}

void CModArt::SetLength(int iLength)
{
	// Set the current value for the m_iLength variable
	m_iLength = iLength;
}

int CModArt::GetSegments(void)
{
	// Return the current value for the m_iSegments variable
	return m_iSegments;
}

void CModArt::SetSegments(int iSegments)
{
	// Set the current value for the m_iSegments variable
	m_iSegments = iSegments;
}
