#include "StdAfx.h"
#include <math.h>
#include "spinner.h"

COLORREF CSpinner::m_crColors[8] = {
	RGB(   0,   0,   0),    // Black
    RGB(   0,   0, 255),    // Blue
    RGB(   0, 255,   0),    // Green
    RGB(   0, 255, 255),    // Cyan
    RGB( 255,   0,   0),    // Red
    RGB( 255,   0, 255),    // Magenta
    RGB( 255, 255,   0),    // Yellow
    RGB( 255, 255, 255)     // White
};

CSpinner::CSpinner(void)
: m_iColor(0)
, m_iMinute(0)
, m_iRadius(0)
, m_ptCenter(0)
, m_pViewWnd(NULL)
, m_pbContinue(NULL)
{
}

CSpinner::~CSpinner(void)
{
}

void CSpinner::Draw(void)
{
    // Get a pointer to the device context
    CDC *pDC = m_pViewWnd->GetDC();
    // Set the mapping mode
    pDC->SetMapMode (MM_LOENGLISH);
    // Copy the spinner center
    CPoint org = m_ptCenter;
    CPoint ptStartPoint;
    // Set the starting point
    ptStartPoint.x = (m_iRadius / 2);
    ptStartPoint.y = (m_iRadius / 2);
    // Set the origination point
    org.x = m_ptCenter.x + (m_iRadius / 2);
    org.y = m_ptCenter.y + m_iRadius;
    // Set the viewport origination point
    pDC->SetViewportOrg(org.x, org.y);

    CPoint ptEndPoint;
    // Calculate the angle of the next line
    double nRadians = (double) (m_iMinute * 6) * 0.017453292;
    // Set the end point of the line
    ptEndPoint.x = (int) (m_iRadius * sin(nRadians));
    ptEndPoint.y = (int) (m_iRadius * cos(nRadians));


    // Create the pen to use
    CPen pen(PS_SOLID, 0, m_crColors[m_iColor]);
    // Select the pen for use
    CPen* pOldPen = pDC->SelectObject(&pen);

    // Move to the starting point
    pDC->MoveTo (ptEndPoint);
    // Draw the line to the end point
    pDC->LineTo (ptStartPoint);

    // Reselect the previous pen
    pDC->SelectObject(&pOldPen);

    // Release the device context
    m_pViewWnd->ReleaseDC(pDC);

    // Increment the minute
    if (++m_iMinute == 60)
    {
        // If the minutes have gone full circle, reset to 0
        m_iMinute = 0;
        // Increment the color
        if (++m_iColor == 8)
            // If we've gone through all colors, start again 
            m_iColor = 0;
    }
}
